package com.persagy.dptool;

import com.persagy.dptool.dto.PhysicalObject;
import com.persagy.dptool.dto.physical.*;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class PhysicalCheckUtil {
    public static final String[] tipsArray = {"无异常。", "无指定项目数据。", "无数据。"};
    public static final String checkPrefix = "check_";

    public String check_projects(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<Project> projects = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    projects.add(CommonUtil.jsonStrToObj(jsonStr, Project.class));
                }
            }
        }catch (Exception e){
            return "数据格式错误！无法转为Json对象。";
        }

        if(projects.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(projects, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_objects(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<Objects> objects = new ArrayList<>();
        Set<String> objIds = new HashSet<>();

        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    Objects item = CommonUtil.jsonStrToObj(jsonStr, Objects.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        objects.add(item);

                        if(item.getObj_id() != null) {
                            objIds.add(item.getObj_id());
                        }
                    }
                }
            }
        }catch (Exception e){
            return "数据格式错误！无法转为Json对象。";
        }


        if(objects.isEmpty()) {
            return tipsArray[1];
        }

        if(!objIds.isEmpty()) {
            physicalObject.objIdSet = new HashSet<>(objIds);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(objects, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_obj_infos(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<ObjectInfo> objectInfos = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    ObjectInfo item = CommonUtil.jsonStrToObj(jsonStr, ObjectInfo.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        objectInfos.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(objectInfos.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(objectInfos, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_graph_instance(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<GraphInstance> graphInstances = new ArrayList<>();
        Set<String> graphIdSet = new HashSet<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    GraphInstance item = CommonUtil.jsonStrToObj(jsonStr, GraphInstance.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        graphInstances.add(item);

                        if(item.getGraph_type() != null && item.getSequence_id() != null) {
                            graphIdSet.add(item.getGraph_type()+item.getSequence_id());
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(graphInstances.isEmpty()) {
            return tipsArray[1];
        }

        if(!graphIdSet.isEmpty()) {
            physicalObject.graphIdSet = new HashSet<>(graphIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(graphInstances, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_rel_btw_graph_period(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<RelBtwGraphPeriod> relBtwGraphPeriods = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    RelBtwGraphPeriod item = CommonUtil.jsonStrToObj(jsonStr, RelBtwGraphPeriod.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        relBtwGraphPeriods.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(relBtwGraphPeriods.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(relBtwGraphPeriods, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_rel_btw_objs(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<RelBtwObjs> relBtwObjs = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    RelBtwObjs item = CommonUtil.jsonStrToObj(jsonStr, RelBtwObjs.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        relBtwObjs.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(relBtwObjs.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(relBtwObjs, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_infocode_scheme(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<InfocodeScheme> infocodeSchemes = new ArrayList<>();
        Set<String> infocodeSchemeIdSet = new HashSet<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    InfocodeScheme item = CommonUtil.jsonStrToObj(jsonStr, InfocodeScheme.class);
                    if(item.getScheme_id() != null) {
                        infocodeSchemeIdSet.add(item.getScheme_id());
                    }
                    infocodeSchemes.add(item);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(infocodeSchemes.isEmpty()) {
            return tipsArray[2];
        }

        if(!infocodeSchemeIdSet.isEmpty()) {
            physicalObject.infoCodeSchemeIdSet = new HashSet<>(infocodeSchemeIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(infocodeSchemes, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_infocode_scheme_follower(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<InfocodeSchemeFollower> infocodeSchemeFollowers = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    InfocodeSchemeFollower item = CommonUtil.jsonStrToObj(jsonStr, InfocodeSchemeFollower.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        infocodeSchemeFollowers.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(infocodeSchemeFollowers.isEmpty()) {
            return tipsArray[2];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(infocodeSchemeFollowers, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }
}
